/*
 * fx2-io -- FX2 CAMAC IO via USB example. 
 * 
 * Based on usb-fx2-local-examples - by Wolfgang Wieser ] wwieser (a) gmx <*> de [ 
 * 
 * This file may be distributed and/or modified under the terms of the 
 * GNU General Public License version 2 as published by the Free Software 
 * Foundation. (See COPYING.GPL for details.)
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 *  Registers - CAMAC
 *
 *  PD0-PD7 - R-lines:  PA4=0 (R1-R8), PA4=1 (R9-R12)
 *
 *  PB0-PB3: A1-A8,  PB4-PB7: F1-F8 PA1: F16
 *
 *  PA0: L(input), PA1: F16, PA2: N2 - PA3: - , PA4: (R-lines - 0:[R1-R8], 1:[R9-R12]), PA5: S2, PA6: Z, PA7: N1  
 */

#define ALLOCATE_EXTERN
#include <fx2regs.h>

// Read TRM p.15-115 for an explanation on this. 
// A single nop is sufficient for default setup but like that we're on 
// the safe side. 
#define	NOP		_asm nop _endasm
#define	SYNCDELAY	NOP; NOP; NOP; NOP
static unsigned char nn=0;
static unsigned char len=0;
xdata unsigned char *dest=EP6FIFOBUF+2;
xdata unsigned char *dest0=EP6FIFOBUF;
static unsigned char dd[50]; 
unsigned char CA=0;
void out(unsigned int);
void rout1(unsigned char);
void rout2(unsigned char);
unsigned int naf1(unsigned char);
unsigned int naf2(unsigned char);
void nafh(unsigned char, unsigned char);
unsigned int naf(unsigned char, unsigned char, unsigned char);
void Z(void);
void event(void);
void delay(unsigned char);

#include <event-56.c>

void out(unsigned int H)
{
  dd[nn++]=(H&0x00ff);
  dd[nn++]=((H>>8)&0x00ff);
}
//unsigned int naf(unsigned char af, unsigned char N){ // for F<16 
unsigned int naf(unsigned char N, unsigned char A, unsigned char F){
 unsigned char af;
 af=(F<<4)|A;
 if(F>15) {
   nafh(af,N);
   return 0;
   }
 IOB=af; //af = FFFFAAAA (F<16)
 if(N==1){IOA=0x80;} //select N1-line: start camac naf
 else{IOA=0x04;} //select N2-line: start camac naf
  CA=IOD;
 // F>2 no data transfer
 IOA=IOA|0x20; //S2
 NOP; NOP;
 IOA=IOA&0xdf; // end of S2
 IOA=0; //end of camac cycle
 return CA;
}

void rout1(unsigned char af){ // for N=1, F=0 or 2 
 IOB=af; //af = FFFFAAAA (F<16)
 IOA=0x80; //select N1-line: start camac naf
 //F=0: data read
 dd[nn++]=IOD;
 IOA=IOA|0x10; // select R9-R12
 dd[nn++]=IOD&0x0f; // only R9-R12 valid
 IOA=0; //end of camac cycle
}

unsigned int naf1(unsigned char af){ // for N=1, F=0 or 2
 union{
  unsigned char B[2];
  unsigned int W;
  }CA;
 IOB=af; //af = FFFFAAAA (F<16)
 IOA=0x80; //select N1-line: start camac naf
 //F=0: data read
 CA.B[0]=IOD;
 IOA=IOA|0x10; // select R9-R12
 CA.B[1]=IOD&0x0f; // only R9-R12 valid
 IOA=0; //end of camac cycle
 return CA.W;
}

void rout2(unsigned char af) // for N=2, F=0 or 2 
{
 IOB=af; //af = FFFFAAAA (F<16)
 IOA=0x04; //select N2-line: start camac naf
 //F=0 or 2: data read
 dd[nn++]=IOD;
 IOA=IOA|0x10; // select R9-R12
 dd[nn++]=IOD&0x0f; // only R9-R12 valid
 IOA=0; //end of camac cycle
}

unsigned int naf2(unsigned char af){ // for N=1, F=0 or 2
 union{
  unsigned char B[2];
  unsigned int W;
  }CA;
 IOB=af; //af = FFFFAAAA (F<16)
 IOA=0x04; //select N2-line: start camac naf
 //F=0: data read
 CA.B[0]=IOD;
 IOA=IOA|0x10; // select R9-R12
 CA.B[1]=IOD&0x0f; // only R9-R12 valid
 IOA=0; //end of camac cycle
 return CA.W;
}

void nafh(unsigned char af, unsigned char N){ // for F>=16

 IOA=0x02; //PA1 = 1 - select F16
 IOB=af; //af = FFFFAAAA
 if(N==1)IOA=IOA|0x80; //select N1-line: start camac naf
 if(N==2)IOA=IOA|0x04; //select N2-line: start camac naf
 NOP;
 IOA=IOA|0x20;NOP; NOP; //S2
 IOA=IOA&0xdf; NOP; // end of S2
 IOA=0; //end of camac cycle
 return; 
}

void Z(){
IOA=0x40; // Z - line
NOP;
IOA=0x60; //Z.S2
NOP; NOP; NOP; //160ns
IOA=0x40; // end of S2
NOP;
IOA=0; // end of Z cycle
}

void delay(unsigned char n) //usecond delay
{
unsigned char i;
for(i=0;i<n;i++)NOP;NOP;NOP;NOP;NOP;NOP;NOP;NOP;NOP;NOP;NOP;NOP; //12NOP=999.8 $
}

static void Initialize(void)
{
	CPUCS=0x10;   // 48 MHz, CLKOUT output disabled. 
	
	IFCONFIG=0xc0;  // Internal IFCLK, 48MHz; A,B as normal ports. 
	SYNCDELAY;
	
	REVCTL=0x03;  // See TRM...
	SYNCDELAY;
	
        EP1INCFG=0x00;
	EP1OUTCFG=0xff;
	EP6CFG=0xe2;  // 1110 0010 (bulk IN, 512 bytes, double-buffered)
	SYNCDELAY;

        PORTACFG=0x00; // bits(0-7) IOA
        OEA=0xfe; //PORTA input (LAM bit 0) and output (W bits 1-7)
	OEB=0xff; //PORTB output (W bits 0-7)	
	OED=0x00; //PORTD input (R)
	IOA=0;
	FIFORESET = 0x80;  SYNCDELAY;  // NAK all requests from host. 
	FIFORESET = 0x82;  SYNCDELAY;  // Reset individual EP (2,4,6,8)
	FIFORESET = 0x84;  SYNCDELAY;
	FIFORESET = 0x86;  SYNCDELAY;
	FIFORESET = 0x88;  SYNCDELAY;
	FIFORESET = 0x00;  SYNCDELAY;  // Resume normal operation.
	EP1OUTBC=0xff; // arm endpoint 1 for OUT (host->device) transfers

        Z(); // Z on module
        naf(2,12,11); //A=12, F=11 clear ADC
        naf(1,12,9); //A=12, F=11 clear TDC
	naf(2,12,26); //A=12, F=26 enable LAM - ADC
}

static void ProcessEP1Data(void)
{
	xdata const unsigned char *src=EP1OUTBUF;
	unsigned char cc;
	cc=*src;
	if(cc==0x53){  //STOP
          Z(); // Z on module
          naf(2,12,11); //A=12, F=11 clear ADC
          naf(1,12,9); //A=12, F=9 clear TDC
          *dest0=len;
          *(dest0+1)=0;
          SYNCDELAY;  EP6BCH=0;
          SYNCDELAY;  EP6BCL=64;
          SYNCDELAY;
	  dest=EP6FIFOBUF+2;
	  dest0=EP6FIFOBUF;
	  len=0;
          }

	if(cc==0x52){  //START
          Z(); // Z on module
          naf(2,12,11); //A=12, F=11 clear ADC
          naf(1,12,9); //A=12, F=9 clear TDC
	  naf(2,12,26); //A=12, F=26 enable LAM - ADC
          }
	
	EP1OUTBC=0xff; // re-arm endpoint 1
}


void main(void)
{
	Initialize();

	for(;;)		
	{
		if (!(EP1OUTCS & (1<<1))) ProcessEP1Data();
		// Wait for the EP6 buffer to become non-full. 
		if(!(EP6CS & (0x08)))
		{
		if((IOA&0x01)==1)event();
		}
	}
}
